# -*- coding: utf8 -*-
import math

from flask import Flask, request, render_template, jsonify, redirect, url_for, abort, flash, g, session
from flask.ext.login import login_user, login_required, logout_user, current_user

from burpui import app, bui, login_manager
from burpui.forms import LoginForm
from burpui.misc.utils import human_readable as _hr
from burpui.misc.backend.interface import BUIserverException

@login_manager.user_loader
def load_user(userid):
    if bui.auth != 'none':
        return bui.uhandler.user(userid)
    return None

@app.route('/login', methods=['POST', 'GET'])
def login():
    form = LoginForm(request.form)
    if form.validate_on_submit():
        user = bui.uhandler.user(form.username.data)
        if user.active and user.login(form.username.data, passwd=form.password.data):
            login_user(user, remember=form.remember.data)
            flash('Logged in successfully', 'success')
            return redirect(request.args.get("next") or url_for('home'))
    return render_template('login.html', form=form, login=True)

@app.route('/logout')
@login_required
def logout():
    logout_user()
    return redirect(url_for('home'))

"""
Here is the API

The whole API returns JSON-formated data
"""

@app.route('/api/running-clients.json')
@login_required
def running_clients():
    """
    API: running_clients
    :returns: a list of running clients
    """
    r = bui.cli.is_one_backup_running()
    return jsonify(results=r)

@app.route('/api/render-live-template', methods=['GET'])
@app.route('/api/render-live-template/<name>')
@login_required
def render_live_tpl(name=None):
    """
    API: render_live_tpl
    :param name: the client name if any. You can also use the GET parameter
    'name' to achieve the same thing
    :returns: HTML that should be included directly into the page
    """
    c = request.args.get('name')
    if not name and not c:
        abort(500)
    if not name:
        name = c
    if name not in bui.cli.running:
        abort(404)
    try:
        counters = bui.cli.get_counters(name)
    except BUIserverException:
        counters = []
    return render_template('live-monitor-template.html', cname=name, counters=counters)

@app.route('/api/live.json')
@login_required
def live():
    """
    API: live
    :returns: the live status of the server
    """
    r = []
    for c in bui.cli.is_one_backup_running():
        s = {}
        s['client'] = c
        try:
            s['status'] = bui.cli.get_counters(c)
        except BUIserverException:
            s['status'] = []
        r.append(s)
    return jsonify(results=r)

@app.route('/api/running.json')
@login_required
def backup_running():
    """
    API: backup_running
    :returns: true if at least one backup is running
    """
    j = bui.cli.is_one_backup_running()
    r = len(j) > 0
    return jsonify(results=r)

@app.route('/api/client-tree.json/<name>/<int:backup>', methods=['GET'])
@login_required
def client_tree(name=None, backup=None):
    """
    WebService: return a specific client files tree
    :param name: the client name (mandatory)
    :param backup: the backup number (mandatory)

    """
    j = []
    if not name or not backup:
        return jsonify(results=j)
    root = request.args.get('root')
    try:
        j = bui.cli.get_tree(name, backup, root)
    except BUIserverException, e:
        err = [[2, str(e)]]
        return jsonify(notif=err)
    return jsonify(results=j)

@app.route('/api/clients-report.json')
@login_required
def clients_report_json():
    """
    WebService: return a JSON with global stats
    """
    j = []
    try:
        clients = bui.cli.get_all_clients()
    except BUIserverException, e:
        err = [[2, str(e)]]
        return jsonify(notif=err)
    cl = []
    ba = []
    for c in clients:
        client = bui.cli.get_client(c['name'])
        if not client:
            continue
        f = bui.cli.status('c:{0}:b:{1}:f:log.gz\n'.format(c['name'], client[-1]['number']))
        cl.append( { 'name': c['name'], 'stats': bui.cli.parse_backup_log(f, client[-1]['number']) } )
        for b in client:
            f = bui.cli.status('c:{0}:b:{1}:f:log.gz\n'.format(c['name'], b['number']))
            ba.append(bui.cli.parse_backup_log(f, b['number'], c['name']))
    j.append( { 'clients': cl, 'backups': sorted(ba, key=lambda k: k['end']) } )
    return jsonify(results=j)

@app.route('/api/client-stat.json/<name>')
@app.route('/api/client-stat.json/<name>/<int:backup>')
@login_required
def client_stat_json(name=None, backup=None):
    """
    WebService: return a specific client detailed report
    """
    j = []
    if not name:
        err = [[1, 'No client defined']]
        return jsonify(notif=err)
    if backup:
        try:
            f = bui.cli.status('c:{0}:b:{1}:f:log.gz\n'.format(name, backup))
        except BUIserverException, e:
            err = [[2, str(e)]]
            return jsonify(notif=err)
        j = bui.cli.parse_backup_log(f, backup)
    else:
        try:
            cl = bui.cli.get_client(name)
        except BUIserverException, e:
            err = [[2, str(e)]]
            return jsonify(notif=err)
        for c in cl:
            f =  bui.cli.status('c:{0}:b:{1}:f:log.gz\n'.format(name, c['number']))
            j.append(bui.cli.parse_backup_log(f, c['number']))
    return jsonify(results=j)

@app.route('/api/client.json/<name>')
@login_required
def client_json(name=None):
    """
    WebService: return a specific client backups overview
    """
    try:
        j = bui.cli.get_client(name)
    except BUIserverException, e:
        err = [[2, str(e)]]
        return jsonify(notif=err)
    return jsonify(results=j)

@app.route('/api/clients.json')
@login_required
def clients():
    """
    WebService: return a JSON listing all clients
    """
    try:
        j = bui.cli.get_all_clients()
    except BUIserverException, e:
        err = [[2, str(e)]]
        return jsonify(notif=err)
    return jsonify(results=j)

"""
Here are some custom filters
"""
@app.template_filter()
def mypad (s):
    """
    Filter: used to pad 0's to backup numbers as in the burp's status monitor
    """
    if not s:
        return '0000000'
    return '{0:07d}'.format(int(s))

@app.template_filter()
def time_human(d):
    s = ''
    seconds = (((d % 31536000) % 86400) % 3600) % 60
    minutes = math.floor((((d % 31536000) % 86400) % 3600) / 60)
    hours   = math.floor(((d % 31536000) % 86400) / 3600)
    if hours > 0:
        s = '%02dH' % hours
    return '%s %02dm %02ds' % (s, minutes, seconds)

@app.template_filter()
def bytes_human(b):
    return '{0:.1eM}'.format(_hr(b))

"""
And here is the main site
"""

@app.route('/live-monitor')
@app.route('/live-monitor/<name>')
@login_required
def live_monitor(name=None):
    """
    Live status monitor view
    """
    if not bui.cli.running:
        flash('Sorry, there are no running backups', 'warning')
        return redirect(url_for('home'))
    return render_template('live-monitor.html', live=True, cname=name)

@app.route('/client-browse/<name>', methods=['GET'])
@app.route('/client-browse/<name>/<int:backup>')
@login_required
def client_browse(name=None, backup=None):
    """
    Browse a specific backup of a specific client
    """
    bkp = request.args.get('backup')
    if bkp and not backup:
        return redirect(url_for('client_browse', name=name, backup=bkp))
    return render_template('client-browse.html', tree=True, backup=True, overview=True, cname=name, nbackup=backup)

@app.route('/client-report/<name>')
@login_required
def client_report(name=None):
    """
    Specific client report
    """
    l = bui.cli.get_client(name)
    if len(l) == 1:
        return redirect(url_for('backup_report', name=name, backup=l[0]['number']))
    return render_template('client-report.html', client=True, report=True, cname=name)

@app.route('/clients-report')
@login_required
def clients_report():
    """
    Global report
    """
    return render_template('clients-report.html', clients=True, report=True)

@app.route('/backup-report/<name>', methods=['GET'])
@app.route('/backup-report/<name>/<int:backup>', methods=['GET'])
@login_required
def backup_report(name=None, backup=None):
    """
    Backup specific report
    """
    if not backup:
        backup = request.args.get('backup')
    return render_template('backup-report.html', client=True, backup=True, report=True, cname=name, nbackup=backup)

@app.route('/client', methods=['GET'])
@app.route('/client/<name>')
@login_required
def client(name=None):
    """
    Specific client overview
    """
    if name:
        c = name
    else:
        c = request.args.get('name')
    if bui.cli.is_backup_running(c):
        return redirect(url_for('live_monitor', name=name))
    return render_template('client.html', client=True, overview=True, cname=c)

@app.route('/')
@login_required
def home():
    """
    Home page
    """
    return render_template('clients.html', clients=True, overview=True)



# -*- coding: utf8 -*-
"""
Burp-UI is a web-ui for burp backup written in python with Flask and
jQuery/Bootstrap
"""
__title__ = 'burp-ui'
__author__ = 'Benjamin SANS (Ziirish)'
__license__ = 'BSD 3-clause'


from flask import Flask
from flask.ext.login import LoginManager
from burpui.server import Server as BurpUI

# First, we setup the app
app = Flask(__name__)

app.config['CFG'] = None

app.secret_key = 'VpgOXNXAgcO81xFPyWj07ppN6kExNZeCDRShseNzFKV7ZCgmW2/eLn6xSlt7pYAVBj12zx2Vv9Kw3Q3jd1266A=='
app.jinja_env.globals.update(isinstance=isinstance,list=list)

# We initialize the core
bui = BurpUI(app)

# And the login_manager
login_manager = LoginManager()
login_manager.init_app(app)
login_manager.login_view = 'login'
login_manager.login_message_category = 'info'

# Then we load our routes
import burpui.routes



# -*- coding: utf8 -*-

from flask_wtf import Form
from wtforms import TextField, PasswordField, BooleanField, validators

class LoginForm(Form):
    username = TextField('Username', [validators.Length(min=2, max=25)])
    password = PasswordField('Password', [validators.Required()])
    remember = BooleanField('Remember me', [validators.Optional()])




# -*- coding: utf8 -*-
import ConfigParser
import sys

g_port = 5000
g_bind = '::'
g_refresh = 15
g_ssl = False
g_sslcert = ''
g_sslkey = ''
g_version = 1
g_auth = 'basic'

class Server:
    def __init__(self, app=None):
        self.init = False
        self.app = app

    def setup(self, conf=None):
        global g_refresh, g_port, g_bind, g_ssl, g_sslcert, g_sslkey, g_version, g_auth
        self.sslcontext = None
        if not conf:
            conf = self.app.config['CFG']

        if not conf:
            raise IOError('No configuration file found')

        config = ConfigParser.ConfigParser({'port': g_port,'bind': g_bind,
                    'refresh': g_refresh, 'ssl': g_ssl, 'sslcert': g_sslcert,
                    'sslkey': g_sslkey, 'version': g_version, 'auth': g_auth})
        with open(conf) as fp:
            config.readfp(fp)
            try:
                self.port = config.getint('Global', 'port')
                self.bind = config.get('Global', 'bind')
                self.vers = config.getint('Global', 'version')
                try:
                    self.ssl = config.getboolean('Global', 'ssl')
                except ValueError:
                    self.app.logger.error("Wrong value for 'ssl' key! Assuming 'false'")
                    self.ssl = False
                self.sslcert = config.get('Global', 'sslcert')
                self.sslkey = config.get('Global', 'sslkey')
                self.auth = config.get('Global', 'auth')
                if self.auth != 'none':
                    try:
                        mod = __import__('burpui.misc.auth.{0}'.format(config.get('Global', 'auth')), fromlist=['UserHandler'])
                        UserHandler = mod.UserHandler
                        self.uhandler = UserHandler(self.app)
                    except Exception, e:
                        self.app.logger.error('Import Exception, module \'%s\': %s', config.get('Global', 'auth'), str(e))
                        sys.exit(1)
                else:
                    # I know that's ugly, but hey, I need it!
                    self.app.login_manager._login_disabled = True
            except ConfigParser.NoOptionError:
                self.app.logger.error("Missing option")

            self.app.config['REFRESH'] = config.getint('UI', 'refresh')

        self.app.logger.info('burp version: %d', self.vers)
        self.app.logger.info('listen port: %d', self.port)
        self.app.logger.info('bind addr: %s', self.bind)
        self.app.logger.info('use ssl: %s', self.ssl)
        self.app.logger.info('sslcert: %s', self.sslcert)
        self.app.logger.info('sslkey: %s', self.sslkey)
        self.app.logger.info('refresh: %d', self.app.config['REFRESH'])

        try:
            mod = __import__('burpui.misc.backend.burp{0}'.format(self.vers), fromlist=['Burp'])
            Client = mod.Burp
            self.cli = Client(self.app, conf=conf)
        except Exception, e:
            self.app.logger.error('Failed loading backend for Burp version %d: %s', self.vers, str(e))
            sys.exit(2)

        self.init = True

    def run(self, debug=False):
        if not self.init:
            self.setup()

        if self.ssl:
            from OpenSSL import SSL
            self.sslcontext = SSL.Context(SSL.SSLv23_METHOD)
            self.sslcontext.use_privatekey_file(self.sslkey)
            self.sslcontext.use_certificate_file(self.sslcert)

        if self.sslcontext:
            self.app.run(host=self.bind, port=self.port, debug=debug, ssl_context=self.sslcontext)
        else:
            self.app.run(host=self.bind, port=self.port, debug=debug)




#!/usr/bin/env python
# -*- coding: utf8 -*-
import sys
import os
import unittest
import urllib2
import pprint
from flask.ext.testing import LiveServerTestCase, TestCase

sys.path.append('{0}/..'.format(os.path.join(os.path.dirname(os.path.realpath(__file__)))))

from burpui import app, bui, login_manager

class BurpuiLiveTestCase(LiveServerTestCase):

	def create_app(self):
		conf = os.path.join(os.path.dirname(os.path.realpath(__file__)), '../burpui.cfg')
		app.config['TESTING'] = True
		app.config['LOGIN_DISABLED'] = True
		app.config['LIVESERVER_PORT'] = 5001
		app.config['CFG'] = conf
		bui.setup(conf)
		login_manager.init_app(app)
		return app

	def setUp(self):
		print '\nBegin Test 1\n'

	def tearDown(self):
		print '\nTest 1 Finished!\n'

	def test_server_is_up_and_running(self):
		response = urllib2.urlopen(self.get_server_url())
		self.assertEqual(response.code, 200)

class BurpuiTestCase(TestCase):

	def setUp(self):
		print '\nBegin Test 2\n'

	def tearDown(self):
		print '\nTest 2 Finished!\n'

	def create_app(self):
		app.config['TESTING'] = True
		app.config['LOGIN_DISABLED'] = True
		bui.cli.port = 9999
		login_manager.init_app(app)
		return app

	def test_some_json(self):
		response = self.client.get('/api/clients.json')
		self.assertNotEquals(response.json, dict(results=[]))


if __name__ == '__main__':
	unittest.main()



# -*- coding: utf8 -*-

"""
The following code is used to convert bytes to be human readable.
It was found on the Internet...
"""

import math
import string

# code from here: http://code.activestate.com/recipes/578323-human-readable-filememory-sizes-v2/
class human_readable( long ):
    """ define a human_readable class to allow custom formatting
        format specifiers supported : 
            em : formats the size as bits in IEC format i.e. 1024 bits (128 bytes) = 1Kib 
            eM : formats the size as Bytes in IEC format i.e. 1024 bytes = 1KiB
            sm : formats the size as bits in SI format i.e. 1000 bits = 1kb
            sM : formats the size as bytes in SI format i.e. 1000 bytes = 1KB
            cm : format the size as bit in the common format i.e. 1024 bits (128 bytes) = 1Kb
            cM : format the size as bytes in the common format i.e. 1024 bytes = 1KB
    """
    def __format__(self, fmt):
        # is it an empty format or not a special format for the size class
        if fmt == "" or fmt[-2:].lower() not in ["em","sm","cm"]:
            if fmt[-1].lower() in ['b','c','d','o','x','n','e','f','g','%']:
                # Numeric format.
                return long(self).__format__(fmt)
            else:
                return str(self).__format__(fmt)

        # work out the scale, suffix and base        
        factor, suffix = (8, "b") if fmt[-1] in string.lowercase else (1,"B")
        base = 1024 if fmt[-2] in ["e","c"] else 1000

        # Add the i for the IEC format
        suffix = "i"+ suffix if fmt[-2] == "e" else suffix

        mult = ["","K","M","G","T","P"]

        val = float(self) * factor
        i = 0 if val < 1 else int(math.log(val, base))+1
        v = val / math.pow(base,i)
        v,i = (v,i) if v > 0.5 else (v*base,i-1)

        # Identify if there is a width and extract it
        width = "" if fmt.find(".") == -1 else fmt[:fmt.index(".")]        
        precis = fmt[:-2] if width == "" else fmt[fmt.index("."):-2]

        # do the precision bit first, so width/alignment works with the suffix
        if float(self) == 0:
            return "{0:{1}f}".format(v, precis)
        t = ("{0:{1}f}"+mult[i]+suffix).format(v, precis) 

        return "{0:{1}}".format(t,width) if width != "" else t




# -*- coding: utf8 -*-
from flask.ext.login import UserMixin
from burpui.misc.auth.interface import BUIhandler, BUIuser

import ConfigParser

class BasicLoader:
    def __init__(self, app=None):
        self.app = app
        self.users = {
                'admin': 'admin'
                }
        conf = self.app.config['CFG']
        c = ConfigParser.ConfigParser()
        with open(conf) as fp:
            c.readfp(fp)
            if c.has_section('BASIC'):
                self.users = {}
                for opt in c.options('BASIC'):
                    self.users[opt] = c.get('BASIC', opt)
                    self.app.logger.info('Loading user: %s', opt)

    def fetch(self, uid=None):
        if uid in self.users:
            return uid

        return None

    def check(self, uid=None, passwd=None):
        return uid in self.users and self.users[uid] == passwd



class UserHandler(BUIhandler):
    def __init__(self, app=None):
        self.basic = BasicLoader(app)
        self.users = {}

    def user(self, name=None):
        if name not in self.users:
            self.users[name] = BasicUser(self.basic, name)
        return self.users[name]



class BasicUser(UserMixin, BUIuser):
    def __init__(self, basic=None, name=None):
        self.active = False
        self.basic = basic
        self.name = name

        res = self.basic.fetch(self.name)

        if res:
            self.id = res
            self.active = True

    def login(self, name=None, passwd=None):
        return self.basic.check(name, passwd)

    def is_active(self):
        return self.active

    def get_id(self):
        return self.id




# -*- coding: utf8 -*-
import re
import socket
import time
import datetime
import ConfigParser

from burpui.misc.utils import human_readable as _hr
from burpui.misc.backend.interface import BUIbackend, BUIserverException

g_burpport = 4972
g_burphost = '127.0.0.1'

class Burp(BUIbackend):
    states = {
        'i': 'idle',
        'r': 'running',
        'c': 'client crashed',
        'C': 'server crashed',
        '1': 'scanning',
        '2': 'backup',
        '3': 'merging',
        '4': 'shuffling',
        '7': 'listing',
        '8': 'restoring',
        '9': 'verifying',
        '0': 'deleting'
        }

    counters = [
        'phase',
        'Total',
        'Files',
        'Files (encrypted)',
        'Metadata',
        'Metadata (enc)',
        'Directories',
        'Softlink',
        'Hardlink',
        'Special files',
        'VSS header',
        'VSS header (enc)',
        'VSS footer',
        'VSS footer (enc)',
        'Grand total',
        'warning',
        'estimated_bytes',
        'bytes',
        'bytes_in',
        'bytes_out',
        'start',
        'path'
       ]

    def __init__(self, app=None, host='127.0.0.1', port=4972, conf=None):
        global g_burpport, g_burphost
        self.app = app
        self.host = host
        self.port = port
        self.running = []
        if conf:
            config = ConfigParser.ConfigParser({'bport': g_burpport, 'bhost': g_burphost})
            with open(conf) as fp:
                config.readfp(fp)
                try:
                    self.port = config.getint('Burp1', 'bport')
                    self.host = config.get('Burp1', 'bhost')
                except ConfigParser.NoOptionError, e:
                    self.app.logger.error(str(e))
                except ConfigParser.NoSectionError, e:
                    self.app.logger.error(str(e))
            self.app.logger.info('burp port: %d', self.port)
            self.app.logger.info('burp host: %s', self.host)

    """
    Utilities functions
    """

    def status(self, query='\n'):
        """
        status connects to the burp status port, ask the given 'question' and
        parses the output in an array
        """
        r = []
        try:
            socket.inet_aton(self.host)
            form = socket.AF_INET
        except socket.error:
            form = socket.AF_INET6
        try:
            if not query.endswith('\n'):
                q = '{0}\n'.format(query)
            else:
                q = query
            s = socket.socket(form, socket.SOCK_STREAM)
            s.connect((self.host, self.port))
            s.send(q)
            s.shutdown(socket.SHUT_WR)
            f = s.makefile()
            s.close()
            for l in f.readlines():
                line = l.rstrip('\n')
                if not line:
                    continue
                r.append(line)
            f.close()
            return r
        except socket.error:
            self.app.logger.error('Cannot contact burp server at %s:%s', self.host, self.port)
            raise BUIserverException('Cannot contact burp server at {0}:{1}'.format(self.host, self.port))

    def parse_backup_log(self, f, n, c=None):
        """
        parse_backup_log parses the log.gz of a given backup and returns a dict
        containing different stats used to render the charts in the reporting view
        """
        lookup_easy = {
                'start':    '^Start time: (.+)$',
                'end':      '^\s*End time: (.+)$',
                'duration': '^Time taken: (.+)$',
                'totsize':  '^\s*Bytes in backup:\s+(\d+)',
                'received': '^\s*Bytes received:\s+(\d+)'
                }
        lookup_complex = {
                'files':         '^\s*Files:\s+(.+)\s+\|\s+(\d+)$',
                'files_enc':     '^\s*Files \(encrypted\):\s+(.+)\s+\|\s+(\d+)$',
                'dir':           '^\s*Directories:\s+(.+)\s+\|\s+(\d+)$',
                'softlink':      '^\s*Soft links:\s+(.+)\s+\|\s+(\d+)$',
                'hardlink':      '^\s*Hard links:\s+(.+)\s+\|\s+(\d+)$',
                'meta':          '^\s*Meta data:\s+(.+)\s+\|\s+(\d+)$',
                'meta_enc':      '^\s*Meta data\(enc\):\s+(.+)\s+\|\s+(\d+)$',
                'special':       '^\s*Special files:\s+(.+)\s+\|\s+(\d+)$',
                'efs':           '^\s*EFS files:\s+(.+)\s+\|\s+(\d+)$',
                'vssheader':     '^\s*VSS headers:\s+(.+)\s+\|\s+(\d+)$',
                'vssheader_enc': '^\s*VSS headers \(enc\):\s+(.+)\s+\|\s+(\d+)$',
                'vssfooter':     '^\s*VSS footers:\s+(.+)\s+\|\s+(\d+)$',
                'vssfooter_enc': '^\s*VSS footers \(enc\):\s+(.+)\s+\|\s+(\d+)$',
                'total':         '^\s*Grand total:\s+(.+)\s+\|\s+(\d+)$'
                }
        backup = { 'windows': False, 'number': int(n) }
        if c is not None:
            backup['name'] = c 
        useful = False
        for line in f:
            if re.match('^\d{4}-\d{2}-\d{2} (\d{2}:){3} \w+\[\d+\] Client is Windows$', line):
                backup['windows'] = True
            elif not useful and not re.match('^-+$', line):
                continue
            elif useful and re.match('^-+$', line):
                useful = False
                continue
            elif re.match('^-+$', line):
                useful = True
                continue

            found = False
            # this method is not optimal, but it is easy to read and to maintain
            for key, regex in lookup_easy.iteritems():
                r = re.search(regex, line)
                if r:
                    found = True
                    if key in ['start', 'end']:
                        backup[key] = int(time.mktime(datetime.datetime.strptime(r.group(1), '%Y-%m-%d %H:%M:%S').timetuple()))
                    elif key == 'duration':
                        tmp = r.group(1).split(':')
                        tmp.reverse()
                        i = 0
                        fields = [0] * 4
                        for v in tmp:
                            fields[i] = int(v)
                            i += 1
                        seconds = 0
                        seconds += fields[0]
                        seconds += fields[1] * 60
                        seconds += fields[2] * (60 * 60)
                        seconds += fields[3] * (60 * 60 * 24)
                        backup[key] = seconds
                    else:
                        backup[key] = int(r.group(1))
                    # break the loop as soon as we find a match
                    break

            # if found is True, we already parsed the line so we can jump to the next one
            if found:
                continue

            for key, regex in lookup_complex.iteritems():
                r = re.search(regex, line)
                if r:
                    self.app.logger.debug("match[1]: '{0}'".format(r.group(1)))
                    sp = re.split('\s+', r.group(1))
                    backup[key] = {
                            'new':       int(sp[0]),
                            'changed':   int(sp[1]),
                            'unchanged': int(sp[2]),
                            'deleted':   int(sp[3]),
                            'total':     int(sp[4]),
                            'scanned':   int(r.group(2))
                            }
                    break
        return backup

    def get_counters(self, name=None):
        """
        get_counters parses the stats of the live status for a given client and
        returns a dict
        """
        r = {}
        if not name or name not in running:
            return r
        f = self.status('c:{0}\n'.format(name))
        if not f:
            return r
        for line in f:
            self.app.logger.debug('line: {0}'.format(line))
            rs = re.search('^{0}\s+(\d)\s+(\S)\s+(.+)$'.format(name), line)
            if rs and rs.group(2) == 'r' and int(rs.group(1)) == 2:
                c = 0
                for v in rs.group(3).split('\t'):
                    self.app.logger.debug('{0}: {1}'.format(self.counters[c], v))
                    if c > 0 and c < 15:
                        val = map(int, v.split('/'))
                        if val[0] > 0 or val[1] > 0 or val[2] or val[3] > 0:
                            r[self.counters[c]] = val
                    else:
                        if 'path' == self.counters[c]:
                            r[self.counters[c]] = v.encode('utf-8')
                        else:
                            r[self.counters[c]] = int(v)
                    c += 1
        diff = time.time() - int(r['start'])
        byteswant = int(r['estimated_bytes'])
        bytesgot = int(r['bytes_in'])
        bytespersec = bytesgot / diff
        bytesleft = byteswant - bytesgot
        r['speed'] = bytespersec
        if (bytespersec > 0):
            timeleft = int(bytesleft / bytespersec)
            r['timeleft'] = timeleft
        else:
            r['timeleft'] = -1
        return r

    def is_backup_running(self, name=None):
        """
        is_backup_running returns True if the given client is currently running a
        backup
        """
        if not name:
            return False
        try:
            f = self.status('c:{0}\n'.format(name))
        except BUIserverException:
            return False
        for line in f:
            r = re.search('^{0}\s+\d\s+(\S)'.format(name), line)
            if r and r.group(1) not in [ 'i', 'c', 'C' ]:
                return True
        return False

    def is_one_backup_running(self):
        """
        is_one_backup_running returns a list of clients name that are currently
        running a backup
        """
        r = []
        try:
            cls = self.get_all_clients()
        except BUIserverException:
            return r
        for c in cls:
            if self.is_backup_running(c['name']):
                r.append(c['name'])
        self.running = r
        return r

    def get_all_clients(self):
        """
        get_all_clients returns a list of dict representing each clients with their
        name, state and last backup date
        """
        j = []
        f = self.status()
        for line in f:
            self.app.logger.debug("line: '{0}'".format(line))
            regex = re.compile('\s*(\S+)\s+\d\s+(\S)\s+(.+)')
            m = regex.search(line)
            c = {}
            c['name'] = m.group(1)
            c['state'] = self.states[m.group(2)]
            infos = m.group(3)
            self.app.logger.debug("infos: '{0}'".format(infos))
            if infos == "0":
                c['last'] = 'never'
            elif re.match('^\d+\s\d+\s\d+$', infos):
                sp = infos.split()
                c['last'] = datetime.datetime.fromtimestamp(int(sp[2])).strftime('%Y-%m-%d %H:%M:%S')
            else:
                sp = infos.split('\t')
                c['last'] = datetime.datetime.fromtimestamp(int(sp[len(sp)-2])).strftime('%Y-%m-%d %H:%M:%S')
            j.append(c)
        return j

    def get_client(self, name=None):
        """
        get_client returns a list of dict representing the backups (with its number
        and date) of a given client
        """
        r = []
        if not name:
            return r
        c = name
        f = self.status('c:{0}\n'.format(c))
        for line in f:
            if not re.match('^{0}\t'.format(c), line):
                continue
            self.app.logger.debug("line: '{0}'".format(line))
            regex = re.compile('\s*(\S+)\s+\d\s+(\S)\s+(.+)')
            m = regex.search(line)
            if m.group(3) == "0" or m.group(2) not in [ 'i', 'c', 'C' ]:
                continue
            backups = m.group(3).split('\t')
            for b in backups:
                ba = {}
                sp = b.split()
                ba['number'] = sp[0]
                ba['date'] = datetime.datetime.fromtimestamp(int(sp[2])).strftime('%Y-%m-%d %H:%M:%S')
                r.append(ba)
        # Here we need to reverse the array so the backups are sorted by date ASC
        r.reverse()
        return r

    def get_tree(self, name=None, backup=None, root=None):
        """
        get_tree returns a list of dict representing files/dir (with their attr)
        within a given path
        """
        r = []
        if not name or not backup:
            return r
        if not root:
            top = ''
        else:
            top = root.encode('utf-8')

        f = self.status('c:{0}:b:{1}:p:{2}\n'.format(name, backup, top))
        useful = False
        for line in f:
            self.app.logger.debug("line: '{0}'".format(line))
            if not useful and re.match('^-list begin-$', line):
                useful = True
                continue
            if useful and re.match('^-list end-$', line):
                useful = False
                continue
            if useful:
                t = {}
                m = re.search('^(.{10})\s', line)
                if m:
                    if re.match('^d', m.group(1)):
                        t['type'] = 'd'
                    else:
                        t['type'] = 'f'
                    sp = re.split('\s+', line)
                    t['mode'] = sp[0]
                    t['inodes'] = sp[1]
                    t['uid'] = sp[2]
                    t['gid'] = sp[3]
                    t['size'] = '{0:.1eM}'.format(_hr(sp[4]))
                    t['date'] = '{0} {1}'.format(sp[5], sp[6])
                    t['name'] = sp[len(sp)-1]
                    t['parent'] = top
                    r.append(t)
        return r



# -*- coding: utf8 -*-

class BUIhandler:
    def __init__(self, app=None):
        pass

    def user(self, name=None):
        raise NotImplementedError("Sorry, the current Backend does not implement this method!")

class BUIuser:
    def login(self, name=None, passwd=None):
        raise NotImplementedError("Sorry, the current Backend does not implement this method!")



# -*- coding: utf8 -*-

class BUIbackend:
    def __init__(self, app=None, host='127.0.0.1', port=4972):
        self.app = app
        self.host = host
        self.port = port

    def status(self, query='\n'):
        raise NotImplementedError("Sorry, the current Backend does not implement this method!")

    def parse_backup_log(self, f, n, c=None):
        raise NotImplementedError("Sorry, the current Backend does not implement this method!")

    def get_counters(self, name=None):
        raise NotImplementedError("Sorry, the current Backend does not implement this method!")

    def is_backup_running(self, name=None):
        raise NotImplementedError("Sorry, the current Backend does not implement this method!")

    def is_one_backup_running(self):
        raise NotImplementedError("Sorry, the current Backend does not implement this method!")

    def get_all_clients(self):
        raise NotImplementedError("Sorry, the current Backend does not implement this method!")

    def get_client(self, name=None):
        raise NotImplementedError("Sorry, the current Backend does not implement this method!")

    def get_tree(self, name=None, backup=None, root=None):
        raise NotImplementedError("Sorry, the current Backend does not implement this method!")

class BUIserverException(Exception):
    pass



# -*- coding: utf8 -*-
from flask.ext.login import UserMixin
from burpui.misc.auth.interface import BUIhandler, BUIuser

try:
    import simpleldap
except ImportError:
    raise ImportError('Unable to load \'simpleldap\' module')

import ConfigParser

class LdapLoader:
    def __init__(self, app=None):
        self.app = app
        conf = self.app.config['CFG']
        c = ConfigParser.ConfigParser({'host': 'localhost', 'binddn': '', 'bindpw': '', 'filter': '', 'base': ''})
        with open(conf) as fp:
            c.readfp(fp)
            try:
                self.host = c.get('LDAP', 'host')
                self.filt = c.get('LDAP', 'filter')
                self.base = c.get('LDAP', 'base')
                self.binddn = c.get('LDAP', 'binddn')
                self.bindpw = c.get('LDAP', 'bindpw')
            except ConfigParser.NoOptionError, e:
                self.app.logger.error(str(e))
            except ConfigParser.NoSectionError, e:
                self.app.logger.error(str(e))

        self.app.logger.info('LDAP host: %s', self.host)
        self.app.logger.info('LDAP filter: %s', self.filt)
        self.app.logger.info('LDAP base: %s', self.base)
        self.app.logger.info('LDAP binddn: %s', self.binddn)
        self.app.logger.info('LDAP bindpw: %s', '*****' if self.bindpw else 'None')

        try:
            self.ldap = simpleldap.Connection(self.host, dn=self.binddn, password=self.bindpw)
            self.app.logger.info('OK, connected to LDAP')
        except:
            self.app.logger.error('Could not connect to LDAP')
            self.ldap = None

    def __exit__(self, exc_type, exc_value, traceback):
        if self.ldap:
            self.ldap.close()

    def fetch(self, uid=None):
        try:
            if self.filt:
                self.app.logger.info('filter: %s | base: %s', self.filt, self.base)
                r = self.ldap.search(self.filt, base_dn=self.base)
                for record in r:
                    if record['uid'][0] == uid:
                        return record['uid'][0]
                return None
            else:
                query = 'uid={0}'.format(uid)
                self.app.logger.info('query: %s | base: %s', query, self.base)
                r = self.ldap.search(query, base_dn=self.base)
        except:
            self.app.logger.info('Ooops, LDAP lookup failed')
            return None

        return r[0]['uid'][0]

    def check(self, uid=None, passwd=None):
        try:
            l = simpleldap.Connection(self.host, dn='uid={0},{1}'.format(uid, self.base), password=passwd)
        except:
            return False

        l.close()
        return True



class UserHandler(BUIhandler):
    def __init__(self, app=None):
        self.ldap = LdapLoader(app)
        self.users = {}

    def user(self, name=None):
        if name not in self.users:
            self.users[name] = LdapUser(self.ldap, name)
        return self.users[name]



class LdapUser(UserMixin, BUIuser):
    def __init__(self, ldap=None, name=None):
        self.active = False
        self.ldap = ldap
        self.name = name

        ldapres = self.ldap.fetch(self.name)

        if ldapres:
            self.id = ldapres
            self.active = True

    def login(self, name=None, passwd=None):
        return self.ldap.check(name, passwd)

    def is_active(self):
        return self.active

    def get_id(self):
        return self.id




#!/usr/bin/env python
# coding: utf-8

import os
import os.path
import re
import sys

from setuptools import setup, find_packages

if sys.version < '3':
    import codecs
    def u(x):
        return codecs.unicode_escape_decode(x)[0]
else:
    def u(x):
        return x

datadir = os.path.join('share', 'burpui', 'etc')

setup(
    name='burp-ui',
    version=open('VERSION').read().rstrip(),
    description=u('Burp-UI is a web-ui for burp backup written in python with Flask and jQuery/Bootstrap'),
    long_description=open('README.rst').read(),
    license=open('LICENSE').read(),
    author=u('Benjamin SANS (Ziirish)'),
    author_email=u('ziirish@ziirish.info'),
    url='http://git.ziirish.me/ziirish/burp-ui',
    keywords='burp web ui',
    packages=find_packages(),
    include_package_data=True,
    package_data={
        'static': 'burpui/static/*',
        'templates': 'burpui/templates/*'
    },
    scripts=['bin/burp-ui'],
    data_files=[(datadir, [os.path.join(datadir, 'burpui.cfg')])],
    install_requires=['Flask==0.10.1', 'Flask-Login==0.2.11', 'Flask-WTF==0.10.0', 'WTForms==2.0.1'],
    extras_require={
        'ldap_authentication': ['simpleldap==0.8']
    },
    classifiers=[
        'Framework :: Flask',
        'Intended Audience :: System Administrators',
        'Natural Language :: English',
        'License :: OSI Approved :: BSD License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: System :: Archiving :: Backup',
        'Topic :: System :: Monitoring'
    ]
)
